Exception: Synthra::Errors::GenerationError

Inherits:
Error
  • Object
show all
Defined in:
lib/synthra/errors.rb

Overview

Base class for errors that occur during data generation

GenerationError and its subclasses are raised when the schema is valid but something goes wrong during the actual data generation process.

Examples:

begin
  schema.generate(count: 1000)
rescue Synthra::GenerationError => e
  puts "Could not generate data: #{e.message}"
end

Constant Summary collapse

ERROR_CODE =
"GENERATION_ERROR"