Module: Synthra::Errors
- Defined in:
- lib/synthra/errors.rb
Overview
Container module for all Synthra error classes
This module provides a namespace for error organization. All errors are also aliased at the Synthra module level for convenience.
Defined Under Namespace
Classes: ApiError, ArraySizeLimitError, ConditionError, ContextError, CycleError, CyclicReferenceError, Error, FieldOrderError, GenerationError, IndentationError, InvalidProbabilityError, LatencyLimitError, MissingFunctionError, MissingReferenceError, MixinError, ParseError, PathError, RecursionLimitError, ReferenceError, ResourceLimitError, ScenarioError, SchemaError, SecurityError, SimulatedBehaviorError, SimulatedConnectionDrop, SimulatedError, SimulatedFailure, SyntaxError, TextLengthLimitError, TypeError, UniquenessError, UnknownTypeError, ValidationError
Instance Attribute Summary collapse
-
#column ⇒ Integer?
readonly
The column number where the error occurred (1-based).
-
#line ⇒ Integer?
readonly
The line number where the error occurred (1-based).
-
#source_context ⇒ Array<String>?
readonly
Lines before/after for context.
-
#source_line ⇒ String?
readonly
The content of the source line with the error.
-
#suggestions ⇒ Array<String>?
readonly
"did you mean" suggestions.
Instance Attribute Details
#column ⇒ Integer? (readonly)
Returns the column number where the error occurred (1-based).
|
|
# File 'lib/synthra/errors.rb', line 77
|
#line ⇒ Integer? (readonly)
Returns the line number where the error occurred (1-based).
|
|
# File 'lib/synthra/errors.rb', line 77
|
#source_context ⇒ Array<String>? (readonly)
Returns lines before/after for context.
|
|
# File 'lib/synthra/errors.rb', line 77
|
#source_line ⇒ String? (readonly)
Returns the content of the source line with the error.
|
|
# File 'lib/synthra/errors.rb', line 77
|
#suggestions ⇒ Array<String>? (readonly)
Returns "did you mean" suggestions.
|
|
# File 'lib/synthra/errors.rb', line 77
|