Exception: Synthra::Errors::GenerationError
- Defined in:
- lib/synthra/errors.rb
Overview
Base class for errors that occur during data generation
GenerationError and its subclasses are raised when the schema is valid but something goes wrong during the actual data generation process.
Direct Known Subclasses
ConditionError, CyclicReferenceError, MissingFunctionError, MissingReferenceError, UniquenessError
Constant Summary collapse
- ERROR_CODE =
"GENERATION_ERROR"